@@ -437,7 +437,7 @@ text_handler           (GMarkupParseContext *context,
       const gchar *range_end;
       gboolean just_saw_marker;
       gssize uline_index = -1;
-      gsize uline_len = -1;
+      gsize uline_len = 0;	/* Quiet GCC */
       
       range_end = NULL;
       range_start = text;
@@ -598,6 +598,8 @@ pango_parse_markup (const char                 *markup_text,
    */
   if (attr_list)
     md->attr_list = pango_attr_list_new ();
+  else
+    md->attr_list = NULL;
 
   md->text = g_string_new ("");
   
