@@ -155,10 +155,12 @@ static guint
 logfont_nosize_hash (gconstpointer v)
 {
   const LOGFONT *lfp = v;
+  gchar *face = g_ascii_strdown (lfp->lfFaceName, -1);
+  guint result = g_str_hash (face) + lfp->lfItalic + lfp->lfWeight;
 
-  return g_str_hash (lfp->lfFaceName) +
-    lfp->lfItalic +
-    lfp->lfWeight;
+  g_free (face);
+
+  return result;
 }
 
 /* Ditto comparison function */
@@ -168,7 +170,7 @@ logfont_nosize_equal (gconstpointer v1,
 {
   const LOGFONT *lfp1 = v1, *lfp2 = v2;
 
-  return (strcmp (lfp1->lfFaceName, lfp2->lfFaceName) == 0
+  return (g_ascii_strcasecmp (lfp1->lfFaceName, lfp2->lfFaceName) == 0
 	  && lfp1->lfItalic == lfp2->lfItalic
 	  && lfp1->lfWeight == lfp2->lfWeight);
 }
@@ -242,7 +244,6 @@ PangoFontMap *
 pango_win32_font_map_for_display (void)
 {
   LOGFONT logfont;
-  RECT rect;
 
   /* Make sure that the type system is initialized */
   g_type_init ();
@@ -263,10 +264,7 @@ pango_win32_font_map_for_display (void)
   pango_win32_font_map_read_aliases (fontmap);
 #endif
 
-  SystemParametersInfo(SPI_GETWORKAREA, 0, &rect, 0);
-  fontmap->resolution =
-    (PANGO_SCALE * 72.27 / 25.4) * ((double) GetDeviceCaps (pango_win32_hdc, HORZSIZE) /
-				    (rect.right - rect.left));
+  fontmap->resolution = PANGO_SCALE / GetDeviceCaps (pango_win32_hdc, LOGPIXELSY) * 72.0;
 
   return PANGO_FONT_MAP (fontmap);
 }
@@ -442,6 +440,8 @@ pango_win32_font_map_load_font (PangoFontMap               *fontmap,
   
   name = g_ascii_strdown (pango_font_description_get_family (description), -1);
 
+  PING(("name=%s", name));
+
   win32family = g_hash_table_lookup (win32fontmap->families, name);
   if (win32family)
     {
@@ -465,6 +465,8 @@ pango_win32_font_map_load_font (PangoFontMap               *fontmap,
 	  GSList *tmp_list = best_match->cached_fonts;
 	  gint size = pango_font_description_get_size (description);
 
+	  PING(("got best match:%s",best_match->logfont.lfFaceName));
+
 	  while (tmp_list)
 	    {
 	      PangoWin32Font *win32font = tmp_list->data;
@@ -485,6 +487,7 @@ pango_win32_font_map_load_font (PangoFontMap               *fontmap,
 	    {
 	      PangoWin32Font *win32font;
 	      
+	      PING((""));
 	      win32font = pango_win32_font_new (fontmap, &best_match->logfont, size);
 	      win32font->fontmap = fontmap;
 	      win32font->entry = best_match;
@@ -722,7 +725,7 @@ pango_win32_insert_font (PangoWin32FontMap *win32fontmap,
   PangoStretch stretch;
   int i;
 
-  PING(("lfp.face=%s,wt=%ld,ht=%ld",lfp->lfFaceName,lfp->lfWeight,lfp->lfHeight));
+  PING(("lfp.face=%s,charset=%d,it=%d,wt=%ld,ht=%ld",lfp->lfFaceName,lfp->lfCharSet,lfp->lfItalic,lfp->lfWeight,lfp->lfHeight));
   
   /* First insert the LOGFONT into the list of LOGFONTs for the typeface name
    */
@@ -840,6 +843,15 @@ pango_win32_insert_font (PangoWin32FontMap *win32fontmap,
       break;
     }
 
+  /* Some other magic names */
+
+  /* Recognize just "courier" for "courier new" */
+  if (g_ascii_strcasecmp (win32face->logfont.lfFaceName, "courier new") == 0)
+    {
+      font_family = pango_win32_get_font_family (win32fontmap, "courier");
+      font_family->font_entries = g_slist_append (font_family->font_entries, win32face);
+      win32fontmap->n_fonts++;
+    }
 }
 
 /* Given a LOGFONT and size, make a matching LOGFONT corresponding to
@@ -872,7 +884,7 @@ pango_win32_make_matching_logfont (PangoFontMap  *fontmap,
   while (tmp_list)
     {
       LOGFONT *tmp_logfont = tmp_list->data;
-      int font_size = tmp_logfont->lfHeight;
+      int font_size = abs (tmp_logfont->lfHeight);
 
       if (size != -1)
 	{
@@ -895,7 +907,7 @@ pango_win32_make_matching_logfont (PangoFontMap  *fontmap,
       /* OK, we have a match; let's modify it to fit this size */
 
       *out = *closest_match;
-      out->lfHeight = (int)((double)size / win32fontmap->resolution + 0.5);
+      out->lfHeight = -(int)((double)size / win32fontmap->resolution + 0.5);
       out->lfWidth = 0;
     }
   else
