@@ -200,66 +200,132 @@ pango_xft_pattern_hash (XftPattern *pattern)
   return hash;
 }
 
+typedef enum {
+  PATTERN_FILE       = 1 << 0,
+  PATTERN_INDEX      = 1 << 1,
+  PATTERN_PIXEL_SIZE = 1 << 3,
+  PATTERN_RGBA       = 1 << 4,
+  PATTERN_ANTIALIAS  = 1 << 5,
+  PATTERN_MINSPACE   = 1 << 6,
+  PATTERN_SPACING    = 1 << 7,
+  PATTERN_CHAR_WIDTH = 1 << 8
+} PatternElement;
+
+typedef struct {
+  char  *file;
+  int    index;
+  double pixel_size;
+  int    rgba;
+  Bool   antialias;
+  Bool   minspace;
+  int    spacing;
+  int    char_width;
+} PatternInfo;
+
+static PatternElement
+get_pattern_info (XftPattern  *pattern,
+		  PatternInfo *info)
+{
+  int i;
+  PatternElement fields = 0;
+  
+  for (i = 0; i < pattern->num; i++)
+    {
+      const char *object = pattern->elts[i].object;
+      XftValue *value;
+
+      if (!pattern->elts[i].values)
+	continue;
+
+      value = &pattern->elts[i].values->value;
+
+      switch (object[0])
+	{
+	case 'a':
+	  if (strcmp (object, XFT_ANTIALIAS) == 0 && value->type == XftTypeBool)
+	    {
+	      info->antialias = value->u.b;
+	      fields |= PATTERN_ANTIALIAS;
+	    }
+	  break;
+	case 'c':
+	  if (strcmp (object, XFT_CHAR_WIDTH) == 0 && value->type == XftTypeInteger)
+	    {
+	      info->char_width = value->u.i;
+	      fields |= PATTERN_CHAR_WIDTH;
+	    }
+	  break;
+	case 'f':
+	  if (strcmp (object, XFT_FILE) == 0 && value->type == XftTypeString)
+	    {
+	      info->file = value->u.s;
+	      fields |= PATTERN_FILE;
+	    }
+	  break;
+	case 'i':
+	  if (strcmp (object, XFT_INDEX) == 0 && value->type == XftTypeInteger)
+	    {
+	      info->index = value->u.i;
+	      fields |= PATTERN_INDEX;
+	    }
+	  break;
+	case 'm':
+	  if (strcmp (object, XFT_MINSPACE) == 0 && value->type == XftTypeBool)
+	    {
+	      info->minspace = value->u.b;
+	      fields |= PATTERN_MINSPACE;
+	    }
+	  break;
+	case 'p':
+	  if (strcmp (object, XFT_PIXEL_SIZE) == 0 && value->type == XftTypeDouble)
+	    {
+	      info->pixel_size = value->u.d;
+	      fields |= PATTERN_PIXEL_SIZE;
+	      break;
+	    }
+	case 'r':
+	  if (strcmp (object, XFT_RGBA) == 0 && value->type == XftTypeInteger)
+	    {
+	      info->rgba = value->u.i;
+	      fields |= PATTERN_RGBA;
+	    }
+	  break;
+	case 's':
+	  if (strcmp (object, XFT_SPACING) == 0 && value->type == XftTypeInteger)
+	    {
+	      info->spacing = value->u.i;
+	      fields |= PATTERN_SPACING;
+	      break;
+	    }
+	  break;
+	}
+    }
+
+  return fields;
+}
+
 gboolean
 pango_xft_pattern_equal (XftPattern *pattern1,
 			 XftPattern *pattern2)
 {
-  char *file1, *file2;
-  int index1, index2;
-  double size1, size2;
-  XftResult res1, res2;
-  int int1, int2;
-  Bool bool1, bool2;
-  
-  XftPatternGetString (pattern1, XFT_FILE, 0, &file1);
-  XftPatternGetString (pattern2, XFT_FILE, 0, &file2);
+  PatternInfo info1;
+  PatternInfo info2;
+  PatternElement elements1, elements2;
 
-  g_assert (file1 != NULL && file2 != NULL);
+  elements1 = get_pattern_info (pattern1, &info1);
+  elements2 = get_pattern_info (pattern2, &info2);
 
-  if (strcmp (file1, file2) != 0)
-    return FALSE;
-  
-  if (XftPatternGetInteger (pattern1, XFT_INDEX, 0, &index1) != XftResultMatch)
-    return FALSE;
-  
-  if (XftPatternGetInteger (pattern2, XFT_INDEX, 0, &index2) != XftResultMatch)
+  if (elements1 != elements2)
     return FALSE;
 
-  if (index1 != index2)
-    return FALSE;
-
-  if (XftPatternGetDouble (pattern1, XFT_PIXEL_SIZE, 0, &size1) != XftResultMatch)
-    return FALSE;
-
-  if (XftPatternGetDouble (pattern2, XFT_PIXEL_SIZE, 0, &size2) != XftResultMatch)
-    return FALSE;
-
-  if (size1 != size2)
-    return FALSE;
-
-  res1 = XftPatternGetInteger (pattern1, XFT_RGBA, 0, &int1);
-  res2 = XftPatternGetInteger (pattern2, XFT_RGBA, 0, &int2);
-  if (res1 != res2 || (res1 == XftResultMatch && int1 != int2))
-    return FALSE;
-  
-  res1 = XftPatternGetBool (pattern1, XFT_ANTIALIAS, 0, &bool1);
-  res2 = XftPatternGetBool (pattern2, XFT_ANTIALIAS, 0, &bool2);
-  if (res1 != res2 || (res1 == XftResultMatch && bool1 != bool2))
-    return FALSE;
-  
-  res1 = XftPatternGetBool (pattern1, XFT_MINSPACE, 0, &bool1);
-  res2 = XftPatternGetBool (pattern2, XFT_MINSPACE, 0, &bool2);
-  if (res1 != res2 || (res1 == XftResultMatch && bool1 != bool2))
-    return FALSE;
-
-  res1 = XftPatternGetInteger (pattern1, XFT_SPACING, 0, &int1);
-  res2 = XftPatternGetInteger (pattern2, XFT_SPACING, 0, &int2);
-  if (res1 != res2 || (res1 == XftResultMatch && int1 != int2))
-    return FALSE;
-
-  res1 = XftPatternGetInteger (pattern1, XFT_CHAR_WIDTH, 0, &int1);
-  res2 = XftPatternGetInteger (pattern2, XFT_CHAR_WIDTH, 0, &int2);
-  if (res1 != res2 || (res1 == XftResultMatch && int1 != int2))
+  if (((elements1 & PATTERN_FILE) &&       strcmp (info1.file, info2.file) != 0) ||
+      ((elements1 & PATTERN_INDEX) &&      info1.index != info2.index) ||
+      ((elements1 & PATTERN_PIXEL_SIZE) && info1.pixel_size != info2.pixel_size) ||
+      ((elements1 & PATTERN_RGBA) &&       info1.rgba != info2.rgba) ||
+      ((elements1 & PATTERN_ANTIALIAS) &&  info1.antialias != info2.antialias) ||
+      ((elements1 & PATTERN_MINSPACE) &&   info1.minspace != info2.minspace) ||
+      ((elements1 & PATTERN_SPACING) &&    info1.spacing != info2.spacing) ||
+      ((elements1 & PATTERN_CHAR_WIDTH) && info1.char_width != info2.char_width))
     return FALSE;
   
   return TRUE;
@@ -381,6 +447,34 @@ _pango_xft_font_map_remove (PangoFontMap *fontmap,
 		       xfont->font_pattern);
 }
 
+static PangoXftFamily *
+create_family (PangoXftFontMap *xfontmap,
+	       const char      *family_name)
+{
+  PangoXftFamily *family = g_object_new (PANGO_XFT_TYPE_FAMILY, NULL);
+  family->fontmap = xfontmap;
+  family->family_name = g_strdup (family_name);
+
+  return family;
+}
+
+static gboolean
+is_alias_family (const char *family_name)
+{
+  switch (family_name[0])
+    {
+    case 'm':
+    case 'M':
+      return (g_ascii_strcasecmp (family_name, "monospace") == 0);
+    case 's':
+    case 'S':
+      return (g_ascii_strcasecmp (family_name, "sans") == 0 ||
+	      g_ascii_strcasecmp (family_name, "serif") == 0);
+    }
+
+  return FALSE;
+}
+
 static void
 pango_xft_font_map_list_families (PangoFontMap           *fontmap,
 				  PangoFontFamily      ***families,
@@ -389,6 +483,7 @@ pango_xft_font_map_list_families (PangoFontMap           *fontmap,
   PangoXftFontMap *xfontmap = PANGO_XFT_FONT_MAP (fontmap);
   XftFontSet *fontset;
   int i;
+  int count;
 
   if (xfontmap->n_families < 0)
     {
@@ -399,9 +494,9 @@ pango_xft_font_map_list_families (PangoFontMap           *fontmap,
 			      XFT_FAMILY,
 			      NULL);
 
-      xfontmap->n_families = fontset->nfont;
-      xfontmap->families = g_new (PangoXftFamily *, xfontmap->n_families);
+      xfontmap->families = g_new (PangoXftFamily *, fontset->nfont + 3); /* 3 standard aliases */
 
+      count = 0;
       for (i = 0; i < fontset->nfont; i++)
 	{
 	  char *s;
@@ -409,15 +504,20 @@ pango_xft_font_map_list_families (PangoFontMap           *fontmap,
 	  
 	  res = XftPatternGetString (fontset->fonts[i], XFT_FAMILY, 0, &s);
 	  g_assert (res == XftResultMatch);
-	  
-	  xfontmap->families[i] = g_object_new (PANGO_XFT_TYPE_FAMILY, NULL);
-	  xfontmap->families[i]->family_name = g_strdup (s);
-	  xfontmap->families[i]->fontmap = xfontmap;
+
+	  if (!is_alias_family (s))
+	    xfontmap->families[count++] = create_family (xfontmap, s);
 	}
 
       XftFontSetDestroy (fontset);
+
+      xfontmap->families[count++] = create_family (xfontmap, "Sans");
+      xfontmap->families[count++] = create_family (xfontmap, "Serif");
+      xfontmap->families[count++] = create_family (xfontmap, "Monospace");
+      
+      xfontmap->n_families = count;
     }
-  
+
   if (n_families)
     *n_families = xfontmap->n_families;
   
@@ -508,69 +608,31 @@ pango_xft_font_map_new_font (PangoFontMap  *fontmap,
   
   if (font)
     {
-      /* Revive fonts from cache */
+      g_object_ref (font);
+
+      /* Revive font from cache */
       if (font->in_cache)
 	pango_xft_font_map_cache_remove (fontmap, font);
-      
-      return (PangoFont *)g_object_ref (G_OBJECT(font));
+
+      return (PangoFont *)font;
     }
   
   return  (PangoFont *)_pango_xft_font_new (fontmap, XftPatternDuplicate (match));
 }
 
-static PangoFont *
-pango_xft_font_map_load_font (PangoFontMap               *fontmap,
-			      PangoContext               *context,
-			      const PangoFontDescription *description)
-{
-  PangoXftFontMap *xfontmap = (PangoXftFontMap *)fontmap;
-  XftPattern *pattern, *match;
-  XftResult res;
-  PangoFont *font = NULL;
-  
-  pattern = pango_xft_make_pattern (description);
-      
-  match = XftFontMatch (xfontmap->display, xfontmap->screen, pattern, &res);
-  
-  XftPatternDestroy (pattern);
-  
-  if (match)
-    {
-      font = pango_xft_font_map_new_font (fontmap, match);
-      XftPatternDestroy (match);
-    }
-  
-  return font;
-}
-
-
-static void
-pango_xft_font_set_free (PangoXftPatternSet *font_set)
-{
-  int i;
-  
-  for (i = 0; i < font_set->n_patterns; i++)
-    XftPatternDestroy (font_set->patterns[i]);
-
-  g_free (font_set);
-}
-
-static PangoFontset *
-pango_xft_font_map_load_fontset (PangoFontMap                 *fontmap,
-				 PangoContext                 *context,
-				 const PangoFontDescription   *desc,
-				 PangoLanguage                *language)
+static PangoXftPatternSet *
+pango_xft_font_map_get_patterns (PangoFontMap               *fontmap,
+				 PangoContext               *context,
+				 const PangoFontDescription *desc)
 {
   PangoXftFontMap *xfontmap = (PangoXftFontMap *)fontmap;
   XftPattern *pattern, *pattern_copy;
   XftPattern *match;
-  int i;
   char *family, *family_res;
   XftResult res;
   int id;
   GPtrArray *array;
   PangoXftPatternSet *patterns;
-  PangoFontsetSimple *simple;
 
   patterns = g_hash_table_lookup (xfontmap->fontset_hash, desc);
 
@@ -640,7 +702,44 @@ pango_xft_font_map_load_fontset (PangoFontMap                 *fontmap,
 			   patterns);
     }
 
+  return patterns;
+}
+
+static PangoFont *
+pango_xft_font_map_load_font (PangoFontMap               *fontmap,
+			      PangoContext               *context,
+			      const PangoFontDescription *description)
+{
+  PangoXftPatternSet *patterns = pango_xft_font_map_get_patterns (fontmap, context, description);
+
+  if (patterns->n_patterns > 0)
+    return pango_xft_font_map_new_font (fontmap, patterns->patterns[0]);
   
+  return NULL;
+}
+
+static void
+pango_xft_font_set_free (PangoXftPatternSet *font_set)
+{
+  int i;
+  
+  for (i = 0; i < font_set->n_patterns; i++)
+    XftPatternDestroy (font_set->patterns[i]);
+
+  g_free (font_set);
+}
+
+
+static PangoFontset *
+pango_xft_font_map_load_fontset (PangoFontMap                 *fontmap,
+				 PangoContext                 *context,
+				 const PangoFontDescription   *desc,
+				 PangoLanguage                *language)
+{
+  PangoFontsetSimple *simple;
+  int i;
+  PangoXftPatternSet *patterns = pango_xft_font_map_get_patterns (fontmap, context, desc);
+	  
   simple = pango_fontset_simple_new (language);
 
   for (i = 0; i < patterns->n_patterns; i++)
@@ -798,6 +897,22 @@ _pango_xft_font_desc_from_pattern (XftPattern *pattern, gboolean include_size)
   return desc;
 }
 
+static PangoFontDescription *
+make_alias_description (PangoXftFamily *xfamily,
+			gboolean        bold,
+			gboolean        italic)
+{
+  PangoFontDescription *desc = pango_font_description_new ();
+
+  pango_font_description_set_family (desc, xfamily->family_name);
+  pango_font_description_set_style (desc, italic ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL);
+  pango_font_description_set_variant (desc, PANGO_VARIANT_NORMAL);
+  pango_font_description_set_weight (desc, bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL);
+  pango_font_description_set_stretch (desc, PANGO_STRETCH_NORMAL);
+
+  return desc;
+}
+
 static PangoFontDescription *
 pango_xft_face_describe (PangoFontFace *face)
 {
@@ -809,6 +924,18 @@ pango_xft_face_describe (PangoFontFace *face)
   XftPattern *match_pattern;
   XftPattern *result_pattern;
 
+  if (is_alias_family (xfamily->family_name))
+    {
+      if (strcmp (xface->style, "Regular") == 0)
+	return make_alias_description (xfamily, FALSE, FALSE);
+      else if (strcmp (xface->style, "Bold") == 0)
+	return make_alias_description (xfamily, TRUE, FALSE);
+      else if (strcmp (xface->style, "Italic") == 0)
+	return make_alias_description (xfamily, FALSE, TRUE);
+      else			/* Bold Italic */
+	return make_alias_description (xfamily, TRUE, TRUE);
+    }
+  
   match_pattern = XftPatternBuild (NULL,
 				   XFT_ENCODING, XftTypeString, "iso10646-1",
 				   XFT_FAMILY, XftTypeString, xfamily->family_name,
@@ -875,6 +1002,17 @@ pango_xft_face_get_type (void)
 /*
  * PangoXFontFamily
  */
+static PangoXftFace *
+create_face (PangoXftFamily *xfamily,
+	     const char     *style)
+{
+  PangoXftFace *face = g_object_new (PANGO_XFT_TYPE_FACE, NULL);
+  face->style = g_strdup (style);
+  face->family = xfamily;
+
+  return face;
+}
+
 static void
 pango_xft_family_list_faces (PangoFontFamily  *family,
 			     PangoFontFace  ***faces,
@@ -887,33 +1025,45 @@ pango_xft_family_list_faces (PangoFontFamily  *family,
     {
       XftFontSet *fontset;
       int i;
-      
-      fontset = XftListFonts (xfontmap->display, xfontmap->screen,
-			      XFT_ENCODING, XftTypeString, "iso10646-1",
-			      XFT_FAMILY, XftTypeString, xfamily->family_name,
-			      XFT_CORE, XftTypeBool, False,
-			      NULL,
-			      XFT_STYLE,
-			      NULL);
-      
-      xfamily->n_faces = fontset->nfont;
-      xfamily->faces = g_new (PangoXftFace *, xfamily->n_faces);
 
-      for (i = 0; i < fontset->nfont; i++)
+      if (is_alias_family (xfamily->family_name))
 	{
-	  char *s;
-	  XftResult res;
+	  xfamily->n_faces = 4;
+	  xfamily->faces = g_new (PangoXftFace *, xfamily->n_faces);
+
+	  i = 0;
+	  xfamily->faces[i++] = create_face (xfamily, "Regular");
+	  xfamily->faces[i++] = create_face (xfamily, "Bold");
+	  xfamily->faces[i++] = create_face (xfamily, "Italic");
+	  xfamily->faces[i++] = create_face (xfamily, "Bold Italic");
+	}
+      else
+	{
+	  fontset = XftListFonts (xfontmap->display, xfontmap->screen,
+				  XFT_ENCODING, XftTypeString, "iso10646-1",
+				  XFT_FAMILY, XftTypeString, xfamily->family_name,
+				  XFT_CORE, XftTypeBool, False,
+				  NULL,
+				  XFT_STYLE,
+				  NULL);
 	  
-	  res = XftPatternGetString (fontset->fonts[i], XFT_STYLE, 0, &s);
-	  if (res != XftResultMatch)
-	    s = "Regular";
+	  xfamily->n_faces = fontset->nfont;
+	  xfamily->faces = g_new (PangoXftFace *, xfamily->n_faces);
 	  
-	  xfamily->faces[i] = g_object_new (PANGO_XFT_TYPE_FACE, NULL);
-	  xfamily->faces[i]->style = g_strdup (s);
-	  xfamily->faces[i]->family = xfamily;
+	  for (i = 0; i < fontset->nfont; i++)
+	    {
+	      char *s;
+	      XftResult res;
+	      
+	      res = XftPatternGetString (fontset->fonts[i], XFT_STYLE, 0, &s);
+	      if (res != XftResultMatch)
+		s = "Regular";
+	      
+	      xfamily->faces[i] = create_face (xfamily, s);
+	    }
+
+	  XftFontSetDestroy (fontset);
 	}
-      
-      XftFontSetDestroy (fontset);
     }
   
   if (n_faces)
